type
  TimeRec  = record
	       Hour, Minute, Second, Sec100: word;
	     end;

{----------------------------------------------------------------------------}
procedure ConcatChar(var s: string; c: char);
{----------------------------------------------------------------------------
  Ultraschnelle Prozedur zum Anhngen eines Zeichens an einen String
  (ersetzt den Aufruf 's:=s+c').
 ----------------------------------------------------------------------------}
inline(
  $58/		 { POP AX		c holen				 }
  $5B/		 { POP BX		Zeiger auf s holen		 }
  $07/		 { POP ES						 }
  $26/$FE/$07/	 { INC BYTE PTR ES:[BX] Stringlnge erhhen		 }
  $26/$02/$1F/	 { ADD BL,ES:[BX]	Zeiger auf Zeichen hinter String }
  $80/$D7/$00/	 { ADC BH,0		Addition auf 16 Bit erweitern	 }
  $26/$88/$07);	 { MOV ES:[BX],AL	Zeichen hinter String schreiben	 }

{----------------------------------------------------------------------------}
procedure UpStr(var s: string);
{----------------------------------------------------------------------------
  Diese Prozedur wandelt den String 's' unter Benutzung der Funktion UpCase
  in Grobuchstaben um (Umlaute werden nicht behandelt).
 ----------------------------------------------------------------------------}
var
  i: byte;
begin
  for i:=1 to length(s) do
    s[i]:=UpCase(s[i]);
end;

{-----------------------------------------------------------------------------}
function AddExt(name: string;
                ext:  string): string;
{-----------------------------------------------------------------------------
  Hinzufgen einer Extension zu einem Pfad/Dateinamen. Sofern dieser schon
  eine Extension besitzt, wird die alte ersetzt.
 -----------------------------------------------------------------------------}
var
  i: byte;
begin
  i:=Length(name);                     { Los geht's beim letzten Zeichen }
  while (i>0) and (name[i]<>'.') do    { So lange zurck, bis Anfang erreicht }
    dec(i);                            { ...oder Punkt gefunden }
  if i=0 then                          { Kein Punkt im ganzen String? }
    i:=Length(name)+1;                 { Dann ganzen String bernehmen und
                                         Extension dranhngen }
  AddExt:=copy(name,1,i-1)+'.'+ext;    { Evtl. alte Ext weg und neue anhngen }
end;

{----------------------------------------------------------------------------}
function ScanNum(var s: string; var p: byte; var c: char): word;
{----------------------------------------------------------------------------
  Diese Funktion versucht, aus dem String 's' ab der Positon 'p' eine Zahl
  (word) einzulesen. Falls dies mglich ist, wird die Zahl zurckgegeben,
  ansonsten der Wert $FFFF.
  Nach der Rckkehr zeigt 'p' auf das erste Zeichen nach der Zahl, in
  'c' wird der Wert diese Zeichens zurckgegeben. Falls die Zahl durch
  das Stringende abgeschlossen worden ist, steht in 'c' ein Leerzeichen, und
  'p' zeigt hinter das Stringende.
 --------------------------------------------------------------------------}
var
  p1: byte;
  n:  word;
  x:  integer;
begin
  p1:=p;			       { Anfangsposition im String merken }
  while (p<=length(s)) and (s[p]>='0') and (s[p]<='9') do
				       { Bis Ende oder Nichtziffer durchgehen }
    inc(p);
  if p1=p then			       { Gar keine Ziffern gefunden? }
    n:=$FFFF			       { Rckgabe: keine Ziffer }
  else
    val(copy(s,p1,p-p1),n,x);	       { Ziffern holen und in Zahl umwandeln }
  ScanNum:=n;
  if p<=length(s) then		       { Zahl nicht am Stringende? }
    c:=s[p]			       { Dann folgendes Zeichen zurck }
  else
    c:=' ';			       { Ansonsten Ende wie durch Leerzeichen }
end;

{----------------------------------------------------------------------------}
function ScanCom(var s: string; var p: byte; var c: char): string;
{----------------------------------------------------------------------------
  Diese Funktion versucht, aus dem String 's' ab der Positon 'p' einen durch
  ein Komma beendenten String zu lesen. Falls dies mglich ist, wird der
  String bis zum Komma zurckgegeben, ansonsten ein Leerstring.
  Nach der Rckkehr zeigt 'p' auf das erste Zeichen nach dem gelesenen String,
  in 'c' wird der Wert diese Zeichens zurckgegeben. Falls der Substring durch
  das Stringende abgeschlossen worden ist, steht in 'c' ein Leerzeichen, und
  'p' zeigt hinter das Stringende von 's'.
 --------------------------------------------------------------------------}
var
  p1: byte;
begin
  p1:=p;			       { Anfangsposition im String merken }
  while (p<=length(s)) and (s[p]<>',') do
				       { Bis Ende oder Komma durchgehen }
    inc(p);
  if p1=p then			       { Gar keine Zeichen gefunden? }
    ScanCom:=''			       { Rckgabe: keine Zeichen }
  else
    ScanCom:=copy(s,p1,p-p1);	       { Gefundene Zeichen zurck }
  if p<=length(s) then		       { Nicht am Stringende? }
    c:=s[p]			       { Dann folgendes Zeichen zurck }
  else
    c:=' ';			       { Ansonsten Ende wie durch Leerzeichen }
end;

{----------------------------------------------------------------------------}
procedure ScanWhite(s: string; var p: byte);
{----------------------------------------------------------------------------
  Diese Prozedur berspringt im String 's' alle whitespace-Zeichen (Leer-
  zeichen, Tabulator) ab der Position 'p'. Nach der Rckkehr zeigt 'p' auf
  das erste Nicht-Leerzeichen.
 ----------------------------------------------------------------------------}
begin
  while (p<=length(s)) and ((s[p]=' ') or (s[p]=#09)) do
				       { Bis Ende oder Nicht-Whitespace }
    inc(p);
end;

{----------------------------------------------------------------------------}
function ScanLabel(var s: string; var p: byte; var c: char): LabelStr;
{----------------------------------------------------------------------------
  Diese Prozedur liest aus dem String 's' ab Position 'p' einen Labelnamen
  ein (gltige Zeichen in 'ValidLabel'). Bei der Rckkehr zeigt 'p' hinter
  das Ende des Labels, 'c' gibt das Trennzeichen nach dem Label an.
 ----------------------------------------------------------------------------}
const
  ValidLabel:set of char=['0'..'9','A'..'Z','a'..'z','@','_','^','[',']'];
var
  lb: string;
begin
  lb:='';			       { Noch ist der Label leer }
  while (p<=length(s)) and (s[p] in ValidLabel) do
    begin			       { Bis Nicht-Labelzeichen/Stringende }
      ConcatChar(lb,s[p]);	       { Label zusammensetzen }
      inc(p);			       { Nchstes Zeichen }
    end;
  if p>length(s) then		       { Ende auerhalb String }
    c:=' '			       { Dann Ende durch Space simulieren }
  else
    c:=s[p];			       { Ansonsten Endzeichen zurckgeben }
  ScanLabel:=copy(lb,1,LabelLen);      { Signifikante Zeichen im Label zurck }
end;

{----------------------------------------------------------------------------}
procedure GetTimeRec(var Time: TimeRec);
{----------------------------------------------------------------------------
  Diese Prozedur fhrt die selbe Aufgabe aus wie 'GetTime' aus 'Dos', gibt
  ihr Ergebnis allerdings in Form eines Records zurck.
 ----------------------------------------------------------------------------}
begin
  with Time do			       { Momentane Uhrzeit holen }
    GetTime(Hour,Minute,Second,Sec100);
end;

{----------------------------------------------------------------------------}
procedure TimeDiff(t1,t2: TimeRec; var tr: TimeRec);
{----------------------------------------------------------------------------
  Diese Funktion liefert die Differenz der beiden Zeiten 't1' und 't2'. Dabei
  wird ber Tagesgrenzen grozgig hinweggegangen (d.h. das Ergebnis ist
  auch korrekt, wenn zwischen 't1' und 't2' einmal Mitternacht liegt.
 ----------------------------------------------------------------------------}
begin
  dec(t1.Sec100,t2.Sec100);	       { Hundertstelsekunden abziehen }
  if t1.Sec100>99 then		       { bertrag? }
    begin
      inc(t1.Sec100,100);	       { Hunderstelstelle korrigieren }
      dec(t1.Second,1);		       { Einen "borgen" }
    end;
  dec(t1.Second,t2.Second);	       { Sekunden abziehen }
  if t1.Second>59 then
    begin
      inc(t1.Second,60);
      dec(t1.Minute,1);
    end;
  dec(t1.Minute,t2.Minute);	       { Minuten abziehen }
  if t1.Minute>59 then
    begin
      inc(t1.Minute,60);
      dec(t1.Hour,1);
    end;
  dec(t1.Hour,	t2.Hour	 );	       { Tage abziehen }
  if t1.Hour>23 then
    inc(t1.Hour,24);
  tr:=t1;			       { Ergebnis zurckgeben }
end;

{----------------------------------------------------------------------------}
procedure WriteInt(v: word);
{----------------------------------------------------------------------------
  Diese Prozedur schreibt die Integerzahl 'v' in die Datei 'f' (file of byte).
  Es wird das bliche "low-byte-first"-Format benutzt.
 ----------------------------------------------------------------------------}
var
  l,h: byte;
begin
  l:=Lo(v);
  h:=Hi(v);
  write(f,l,h);
end;

{----------------------------------------------------------------------------}
procedure WriteByt(v: byte);
{----------------------------------------------------------------------------
  Diese Prozedur sieht etwas widersinnig aus, ist aber trotzdem ntzlich, da
  der Compiler gewisse Probleme mit dem Schreiben von Konstanten in typi-
  sierte Dateien hat. Sie legt 'v' in der Datei 'f' (file of byte) ab.
 ----------------------------------------------------------------------------}
begin
  write(f,v);
end;

{----------------------------------------------------------------------------}
procedure WriteTrip(v: PageRec);
{----------------------------------------------------------------------------
  Der Vollstndigkeit halber: Diese Prozedur schreibt die 3-Byte-Zahl 'v' in
  die Datei f (file of byte). Dieses Zahlenformat wird fr das Seitenver-
  zeichnis benutzt.
 ----------------------------------------------------------------------------}
begin
  write(f,v.l,v.m,v.h);
end;
